//----------------------------------------------------------------------------
// Programme emetteur
//----------------------------------------------------------------------------
// Pilotage de la porteuse d'un emetteur à lampe par un arduino
// en utilisant un circuit AD9833
//
// Utilisation d'un récepteur FM pour retransmettre le son stations
// recues sur la FM, afin de les ré-émettre en GO /PO en utilisant
// un circuit SI4703
//
// Fonctions prévues au 2020/06/27
//   - réglage manuel fréquence emission
//   - réglage manuel de la fréquence de réception FM
//   - mémorisation de l'association de ces 2 fréquences dans des presets
//   - gestion de 10 presets
//   - affichage sur LCD, nom de station RDS, fréquence FM, fréquence AM
//   - reglage avec un encodeur rotatif
//	 - utilisation des interruptions pour lire les messages RDS (si possible...)
// Fonctions installees au 2020/11/22
//   - reglage frequence reception FM
//   - reglage frequence emission AM
//   - selection du bobinage antenne : GO+PO en série, ou uniquement GO ou PO
//   - reglage du volume
//   - affichage du RDS non bloquant
//   - lecture/ecriture presets (16 presets en tout)
//   - memorisation du dernier decodage RDS 
//     (pour afficher quand même si station deja decodee et memorisee)
//   - ajout increment fin pour frequence emetteur par un appui lent sur interrupteur
//----------------------------------------------------------------------------




#include <Wire.h>
#include "SI4703.h"
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>
#include <math.h>

// #if defined(ARDUINO) && ARDUINO >= 100
// #define printByte(args)  write(args);
// #else
// #define printByte(args)  print(args,BYTE);
// #endif



// int resetPin = 2;											// reset pin for SI4703
//int SDIO = A4;												// SDA/A4 on Arduino
// int SCLK = A5;												// SCL/A5 on Arduino


char printBuffer[50];
uint16_t si4703_registers[16]; 								// 16 registers, each 16 bits large

						
uint8_t tempbyte;											// for debug
char StationName[10];										// Buffer for current Station Name 
char StationLastName[10];									// Buffer for last Station Name decoded
uint8_t StationNameStatus=0;								// Station name decode status
boolean completedSName[] = {false, false, false, false};	// 4 parts status of the Station Name
uint16_t StationNameCycles=0;								// number of cycle of the decoder
#define MAX_SN_CYCLES 50									// max decode cycles for Station Name
int completedCountSN = 0;									// parts of the Station Name completed
#define SN_PARTS	4										// Number of station name parts
char RadiotextMess[70];										// Buffer for Radiotext message
char RadiotextLastMess[70];									// Buffer for Last Radiotext message (to compare if change occurs)
uint8_t RadiotextStatus=0;									// Radiotext decode status
boolean completedRadiotext[] = {							// 16 parts status of the Radiotext message
	false, false, false, false,
	false, false, false, false,
	false, false, false, false,
	false, false, false, false
};
uint16_t RadiotextCycles=0;									// number of cycle of the decoder for Radiotext
#define MAX_RT_CYCLES 500									// max decode cycles for Radiotext messages
int completedCountRT = 0;									// parts of the Radiotext message completed
#define RT_PARTS	16										// Number of radiotext message parts

uint16_t blockA,blockB,blockC,blockD;						// Local word to store bloc A to D
uint8_t GroupType;											// Last groupe type
int indexSN, indexRT;										// Station name index, radiotext index
char KeyboardInput;											// input char of Serial input
int currentChannel = 880; 									// Defaut tuning
byte currentAntennaCoil = 0;								// Antenna coil commutation
int TuningAntennaCoil = 100;								// CV position antenna coil tuning
byte currentVolumeSI4703 = 10;								// default volume 5 (must be between 0-15)

// PRESETS
//--------
#define MAXPRESETS	16			// Max number of presets
#define PRESETSIZE	16			// Byte size of one preset
byte myByte1,myByte2;	
int myaddress=0;
byte currentPreset=1;			// current preset
uint8_t initEEPROM = 0;			// initialization EEPROM flag

// ENCODERS & SWITCHES
//---------------------
int pinA[] = {5};               	// Pins A encoder
int pinB[] = {4};               	// Pins B encoder
#define PIN_ENCODER_SW	3			// PIN for encoder switch	
#define PIN_COIL1		6			// PIN for Coil1 short-circuit
#define PIN_COIL2		7			// PIN for Coil2 short-circuit
volatile int lastEncoded[] = {0,0}; // Dernieres valeur lue encodeurs
volatile int counter[] = {0,0};     // Valeur courante des encodeurs
int bcounter[] = {0,0};             // valeurs encodeurs divisees par 4
int lcounter[] = {0,0};             // valeurs précédentes des encodeurs
byte valbouton;                     // Numero du bouton appuye
byte lastBoutonNav=0;				// Stockage du dernier etat dun bouton
#define NB_ENCODEURS 1
#define ENC1DOWN	1				// Encodeur 1 down
#define ENC1UP		11				// Encodeur 1 up
#define B_ENC1   1     				// Numero bouton encodeur 1
#define B_MAINT  9					// Bouton encodeur maintenu appuyé
byte currentMenu=0;					// current menu (0=top of the tree);
#define M_PRESETS_CHG   0			// Change preset menu
#define M_RECEIVE_FREQ  1			// Tuning receiver menu
#define M_TRANSMIT_FREQ 2			// Tuning transmitter menu
#define M_COIL          3			// Coil selection menu
#define M_VOLUME        4			// Volume setting menu



unsigned int tcnt1;               	// Sauvegarde valeur counter 1
byte encodeur;
byte FreqTransmitInc=0;				// Increment Frequency Transmitter
long timeFreqChange=0;				// Last time we change frequency
#define DELAY_CHANNEL_CHANGE 1000	// Delay in ms before really changing frequency of SI4703

// LCD
//----
LiquidCrystal_I2C lcd(0x27,20,4);  							// LCD address 0x27, 20 chars, 4 lines
uint8_t symbup[8]  = {0x4,0xe,0xf5,0x4,0x4,0x4,0x4};    	// up arrow char
uint8_t symbdown[8]  = {0x4,0x4,0x4,0x4,0xf5,0xe,0x4};  	// down arrow char
uint8_t symbTriRight[8]  = {0x8,0xc,0xe,0xf,0xe,0xc,0xc8};  // tri char right
uint8_t symbTriLeft[8]  = {0x2,0x6,0xe,0x1e,0xe,0x6,0x2};  	// tri char down
uint8_t symbStop[8]  = {0x0,0x1f,0x1f,0x1f,0x1f,0x1f,0x0};	// stop char
uint8_t symbBarre[8]  = {0x4,0x4,0x4,0x4,0x4,0x4,0x4,0x4};	// separation char
uint8_t symbAntenna[8] = {0x15,0x15,0x15,0x0E,0x04,0x04,0x04,0x04}; // Antenna char
#define CHAR_RIGHT   126
#define CHAR_LEFT    127
#define CHAR_UP        0
#define CHAR_DOWN      1
#define CHAR_TRIRIGHT  2
#define CHAR_TRILEFT   3
#define CHAR_STOP      4
#define CHAR_BARRE     5
#define CHAR_ANTENNA   6


// AD9833
//-------
#define AD_SINE_WAVE        0x2000			
#define AD_TRIANGLE_WAVE    0x2002
#define AD_SQUARE_WAVE      0x2028
#define AD_HALF_SQUARE_WAVE 0x2020
long timeFreqTransmitterChange = 0;			// Last time we change frequency of AD9833
int currentChannelTransmitter = 200;		// Defaut transmitter frequency in Khz
#define DELAY_TRANSMITTER_CHANGE 500		// Delay in ms before really changing frequency of SI4703

// Antenna tuner
long timeAntennaTuningChange = 0;			// Last time we change the tuning of the antenna
#define DELAY_ANTENNA_CHANGE 500			// Delay in ms before really changing tuning of the antenna

// Coil
long timeCoilChange = 0;					// Last time we change the coil
#define DELAY_COIL_CHANGE 100				// Delay in ms before really changing selection of coil

void setup() {                

	pinMode(13, OUTPUT);
	pinMode(PIN_COIL1, OUTPUT);						// First coil short-circuit
	pinMode(PIN_COIL2, OUTPUT);						// Second coil short-circuit
	// Serial.begin(115200);
	// Serial.println();
	
	si4703_init(); 									// Init the Si4703
	ad9833_init();									// Init the AD9833 (also SPI...)

	init_encoders();								// Encoders init
	
	initEEPROM=0;							// Reset flag initialisation eeprom
	valbouton=readButtons();				// Lire les interrupteurs des encodeurs
	if (valbouton==B_ENC1)					// If button press during init
		initEEPROM=1;						// perhaps we have to check if format is necessary
	init_lcd();
	valbouton=readButtons();				// Lire les interrupteurs des encodeurs
	if (valbouton==B_MAINT && initEEPROM)	// If button always pressed 
		FormatEEPROM();						// we format the EEPROM		
	LoadPreset(1);							// load default preset 

	// gotoChannel(currentChannel);										// Set default channel and tune
	// setVolume(currentVolumeSI4703);									// Set default volume level
	// SG_frequencySet(currentChannelTransmitter*1000L,AD_SINE_WAVE);	// Set frequency of the transmitter
	// setTuningAntennaCoil();											// Set tuning of the antenna coil
		
	init_background();						// display background
	refresh_display();						// display data
}

void loop() {
	while(1) {
		menu_main();
	}
}




 

 
 
 


